@@ -83,6 +83,7 @@ hrm.test.matrices <- function(data, alpha=0.05){
   output$subject <- NULL
   output$factors <- list(NULL, NULL)
   output$data <- data
+  output$nonparametric <- FALSE
   class(output) <- "HRM"
   
   return (output)
@@ -101,12 +102,14 @@ hrm.test.matrices <- function(data, alpha=0.05){
 #' @param formula formula object from the user input
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = rep(1,4), formula ){
+hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = rep(1,4), formula, nonparametric ){
+  
+  ranked <- NULL
 
-  temp0 <- if(testing[1]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "A", paste(as.character(group), " (weighted)"))}
-  temp1 <- if(testing[2]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "Au", paste(as.character(group)))}
-  temp2 <- if(testing[3]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "B", paste(as.character(factor1)))}
-  temp3 <- if(testing[4]) {hrm.1w.1f(X, alpha, group , factor1, subject, data, "AB", paste(as.character(group), ":",as.character(factor1)))}
+  temp0 <- if(testing[1]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "A", paste(as.character(group), " (weighted)"), nonparametric, ranked)}
+  temp1 <- if(testing[2]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "Au", paste(as.character(group)), nonparametric, ranked)}
+  temp2 <- if(testing[3]) {hrm.1w.1f(X, alpha, group , factor1,  subject, data, "B", paste(as.character(factor1)), nonparametric, ranked)}
+  temp3 <- if(testing[4]) {hrm.1w.1f(X, alpha, group , factor1, subject, data, "AB", paste(as.character(group), ":",as.character(factor1)), nonparametric, ranked)}
   
   output <- list()
   output$result <- rbind(temp0, temp1, temp2, temp3)
@@ -115,6 +118,7 @@ hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = r
   output$subject <- subject
   output$factors <- list(c(group), c(factor1))
   output$data <- X
+  output$nonparametric <- nonparametric
   class(output) <- "HRM"
   
   return (output)
@@ -133,13 +137,15 @@ hrm.test.2.one <- function(X, alpha, group , factor1, subject, data, testing = r
 #' @param formula formula object from the user input
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data, testing = rep(1,7), formula ){
+hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data, testing = rep(1,7), formula, nonparametric ){
   
+  ranked <- NULL
+
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:7){
     if(testing[i]) {
-      temp[[i]] <- hrm.1w.2f(X, alpha, group , factor1, factor2, subject, data, H = i )
+      temp[[i]] <- hrm.1w.2f(X, alpha, group , factor1, factor2, subject, data, H = i, "", nonparametric, ranked )
     }
   }
   
@@ -150,6 +156,7 @@ hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data,
   output$subject <- subject
   output$factors <- list(c(group), c(factor1, factor2))
   output$data <- X
+  output$nonparametric <- nonparametric
   class(output) <- "HRM"
   
   return (output)
@@ -170,13 +177,15 @@ hrm.test.2.within <- function(X, alpha, group , factor1, factor2, subject, data,
 #' @param formula formula object from the user input
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data, testing = rep(1,7), formula ){
+hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data, testing = rep(1,7), formula, nonparametric ){
 
+  ranked <- NULL
+  
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:7){
     if(testing[i]) {
-      temp[[i]] <- hrm.2w.1f(X, alpha, group , subgroup, factor, subject, data, H = i )
+      temp[[i]] <- hrm.2w.1f(X, alpha, group , subgroup, factor, subject, data, H = i, "", nonparametric, ranked )
     }
   }
   
@@ -187,6 +196,7 @@ hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data
   output$subject <- subject
   output$factors <- list(c(group, subgroup), c(factor))
   output$data <- X
+  output$nonparametric <- nonparametric
   class(output) <- "HRM"
  
   return (output)
@@ -207,13 +217,15 @@ hrm.test.2.between <- function(X, alpha, group , subgroup, factor, subject, data
 #' @param formula formula object from the user input
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, factor2, subject, data, testing = rep(1,15), formula ){
+hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, factor2, subject, data, testing = rep(1,15), formula, nonparametric ){
+  
+  ranked <- NULL
   
   # create list for storing results; NULL used, because it is ignored by rbind
   temp <- list(NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)
   for(i in 1:15){
     if(testing[i]) {
-      temp[[i]] <- hrm.2w.2f(X, alpha, group , subgroup, factor1, factor2, subject, data, H = i )
+      temp[[i]] <- hrm.2w.2f(X, alpha, group , subgroup, factor1, factor2, subject, data, H = i, "", nonparametric, ranked )
     }
   }
   output <- list()
@@ -223,6 +235,7 @@ hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, facto
   output$subject <- subject
   output$factors <- list(c(group, subgroup), c(factor1, factor2))
   output$data <- X
+  output$nonparametric <- nonparametric
   class(output) <- "HRM"
   return (output)
 }
@@ -242,22 +255,25 @@ hrm.test.2.between.within <- function(X, alpha, group , subgroup, factor1, facto
 #' @param formula formula object from the user input
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subject, data, testing = rep(1,15), formula ){
-  temp0 <- if(testing[1]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3, subject, data, P, J, J, J,  paste(as.character(group) ) )}
-  temp1 <- if(testing[2]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, P, J, J, paste(as.character(factor1)) )}
-  temp2 <- if(testing[3]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, J, P, J, paste(as.character(factor2)) )}
-  temp3 <- if(testing[4]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, J, J, P, paste(as.character(factor3)) )}
-  temp4 <- if(testing[5]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, P, J, J, paste(as.character(group),":",as.character(factor1)) )} 
-  temp5 <- if(testing[6]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, J, P, J, paste(as.character(group),":",as.character(factor2)) )}
-  temp6 <- if(testing[7]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, J, J, P, paste(as.character(group),":",as.character(factor3)) )}
-  temp7 <- if(testing[8]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, P, P, J, paste(as.character(factor1),":",as.character(factor2)) )}
-  temp8 <- if(testing[9]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, P, J, P, paste(as.character(factor1),":",as.character(factor3)) )}
-  temp9 <- if(testing[10]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, J, P, P,paste(as.character(factor2),":",as.character(factor3)) )}
-  temp10 <- if(testing[11]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, P, P, J, paste(as.character(group),":",as.character(factor1), ":", as.character(factor2)) )}
-  temp11 <- if(testing[12]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, P, J, P, paste(as.character(group),":",as.character(factor1), ":", as.character(factor3)) )}
-  temp12 <- if(testing[13]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, J, P, P, paste(as.character(group),":",as.character(factor2), ":", as.character(factor3)) )}
-  temp13 <- if(testing[14]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, J, P, P, P, paste(as.character(factor1),":",as.character(factor2), ":", as.character(factor3)) )}
-  temp14 <- if(testing[15]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, P, P, P, P, paste(as.character(group),":",as.character(factor1), ":", as.character(factor2), ":", as.character(factor3)) )}
+hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subject, data, testing = rep(1,15), formula, nonparametric ){
+  
+  ranked <- NULL
+  
+  temp0 <- if(testing[1]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3, subject, data, "P", "J", "J", "J",  paste(as.character(group) ), nonparametric, ranked )}
+  temp1 <- if(testing[2]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "J", paste(as.character(factor1)), nonparametric, ranked )}
+  temp2 <- if(testing[3]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "J", paste(as.character(factor2)), nonparametric, ranked )}
+  temp3 <- if(testing[4]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "J", "P", paste(as.character(factor3)), nonparametric, ranked )}
+  temp4 <- if(testing[5]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "J", paste(as.character(group),":",as.character(factor1)), nonparametric, ranked )} 
+  temp5 <- if(testing[6]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "J", paste(as.character(group),":",as.character(factor2)), nonparametric, ranked )}
+  temp6 <- if(testing[7]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "J", "P", paste(as.character(group),":",as.character(factor3)), nonparametric, ranked )}
+  temp7 <- if(testing[8]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "J", paste(as.character(factor1),":",as.character(factor2)), nonparametric, ranked )}
+  temp8 <- if(testing[9]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "J", "P", paste(as.character(factor1),":",as.character(factor3)), nonparametric, ranked )}
+  temp9 <- if(testing[10]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "J", "P", "P",paste(as.character(factor2),":",as.character(factor3)), nonparametric, ranked )}
+  temp10 <- if(testing[11]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "J", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2)), nonparametric, ranked )}
+  temp11 <- if(testing[12]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "J", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor3)), nonparametric, ranked )}
+  temp12 <- if(testing[13]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "J", "P", "P", paste(as.character(group),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
+  temp13 <- if(testing[14]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "J", "P", "P", "P", paste(as.character(factor1),":",as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
+  temp14 <- if(testing[15]) {hrm.1w.3f(X, alpha, group, factor1, factor2, factor3,  subject, data, "P", "P", "P", "P", paste(as.character(group),":",as.character(factor1), ":", as.character(factor2), ":", as.character(factor3)), nonparametric, ranked )}
   
   output <- list()
   output$result <- rbind(temp0, temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp10, temp11, temp12, temp13, temp14)
@@ -266,6 +282,7 @@ hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subj
   output$subject <- subject
   output$factors <- list(c(group), c(factor1, factor2, factor3))
   output$data <- X
+  output$nonparametric <- nonparametric
   class(output) <- "HRM"
   return (output)
 }
@@ -286,7 +303,7 @@ hrm.test.3.between <- function(X, alpha, group , factor1, factor2, factor3, subj
 #' @return \item{factors}{A list containing the whole- and subplot factors.}
 #' @return \item{data}{The data.frame or list containing the data.}
 #' @keywords internal
-hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
+hrm_test_internal <- function(formula, data, alpha = 0.05,  subject, nonparametric ){
 
   if(missing(data) || !is.data.frame(data)){
     stop("dataframe needed")
@@ -351,8 +368,8 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
   if(length(wholeplot)>2){
     stop("Too many factors are used! Only two wholelot-factors are supported.")
   }
-  if(length(subplot)>3){
-    stop("Too many factors are used! Only three subplotlot-factors are supported.")
+  if(length(subplot)>6){
+    stop("Too many factors are used! Only five subplotlot-factors are supported.")
   }
   if(length(wholeplot)>1 & length(subplot)==3){
     stop("Too many factors are used! Only one whole- and three subplot-factors are supported.")
@@ -360,8 +377,8 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
   if(length(subplot)<1 & length(wholeplot)>1){
     stop("The model needs at least one within-subject factor.")
   }
-  if(length(subplot)>1 & length(wholeplot)<1){
-    stop("The model supports only one subplot-factor when using no wholeplot-factors.")
+  if(length(subplot)>5 & length(wholeplot)<1){
+    stop("The model supports up to five subplot-factor when using no wholeplot-factors.")
   }
   
   # Case: no wholeplot, one subpot factor
@@ -372,10 +389,109 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     subplot<-colnames(dat2[,subplot])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.1.one(X, alpha , factor1, subject, data, formula ))   
+    return(hrm.test.1.one(X, alpha , factor1, subject, data, formula, nonparametric ))   
     
   }
   
+  # Case: no wholeplot, two subpot factor
+  if(length(wholeplot) < 1 & length(subplot) == 2){
+    factor1 <- colnames(dat2)[subplot[1]]
+    factor2 <- colnames(dat2)[subplot[2]]
+    x<-attributes(terms.formula(formula))$term.labels
+    
+    testing <- rep(0,3)
+    for(i in 1:length(x)){
+      
+      tmp <- strsplit(x[i],":")
+      l <- length(tmp[[1]])
+      
+      # interaction hypothesis of 2 factors
+      if(l == 2){testing[3]<-1}
+      else{
+        if(factor1 == x[i]){
+          testing[1]<-1
+        }
+        else if(factor2 == x[i]){testing[2]<-1}
+      }
+    }
+    
+    subplot<-colnames(dat2[,subplot])
+    X<-data
+    data <- colnames(dat)[1]
+    return(hrm.test.2.two(X, alpha , factor1, factor2, subject, data, formula, testing, nonparametric ))   
+    
+  }
+  
+  # Case: no wholeplot, three subpot factor
+  if(length(wholeplot) < 1 & length(subplot) == 3){
+    factor1 <- colnames(dat2)[subplot[1]]
+    factor2 <- colnames(dat2)[subplot[2]]
+    factor3 <- colnames(dat2)[subplot[3]]
+    x<-attributes(terms.formula(formula))$term.labels
+    
+    testing <- rep(0,2^3-1)
+    testing[1:3] <- 1
+    for(i in 1:length(x)){
+      
+      tmp <- strsplit(x[i],":")
+      l <- length(tmp[[1]])
+      
+      # interaction hypothesis of 2 factors
+      if(l == 2){ 
+        if(grepl(factor1,x[i])){
+          if(grepl(factor2,x[i])){
+            testing[4] <- 1
+          } else {
+            testing[5] <- 1
+          }
+        } else {
+          testing[6] <- 1
+        }
+      }
+      
+      # interaction hypothesis of 3 factors
+      if(l == 3){
+        testing[7] <- 1
+      }
+    }
+    
+    subplot<-colnames(dat2[,subplot])
+    X<-data
+    data <- colnames(dat)[1]
+    return(hrm.test.3.three(X, alpha , factor1, factor2, factor3, subject, data, formula, testing, nonparametric ))   
+    
+  }
+  
+  
+  # Case: no wholeplot, four subpot factor
+  if(length(wholeplot) < 1 & length(subplot) == 4){
+    factor1 <- colnames(dat2)[subplot[1]]
+    factor2 <- colnames(dat2)[subplot[2]]
+    factor3 <- colnames(dat2)[subplot[3]]
+    factor4 <- colnames(dat2)[subplot[4]]
+    x<-attributes(terms.formula(formula))$term.labels
+    subplot<-colnames(dat2[,subplot])
+    X<-data
+    data <- colnames(dat)[1]
+    return(hrm.test.4.four(X, alpha , factor1, factor2, factor3, factor4, subject, data, formula, testing = rep(1,2^4-1), nonparametric ))   
+    
+  }
+
+  # Case: no wholeplot, five subpot factor
+  if(length(wholeplot) < 1 & length(subplot) == 5){
+    factor1 <- colnames(dat2)[subplot[1]]
+    factor2 <- colnames(dat2)[subplot[2]]
+    factor3 <- colnames(dat2)[subplot[3]]
+    factor4 <- colnames(dat2)[subplot[4]]
+    factor5 <- colnames(dat2)[subplot[5]]
+    x<-attributes(terms.formula(formula))$term.labels
+    subplot<-colnames(dat2[,subplot])
+    X<-data
+    data <- colnames(dat)[1]
+    return(hrm.test.5.five(X, alpha , factor1, factor2, factor3, factor4, factor5, subject, data, formula, testing = rep(1,2^5-1), nonparametric ))   
+    
+  }  
+  
   # Case: one wholeplot, no subpot factor
   if(length(wholeplot) == 1 & length(subplot) < 1){
     group <- colnames(dat2)[wholeplot[1]]
@@ -384,7 +500,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     wholeplot<-colnames(dat2[,group])
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.1.none(X, alpha , group, subject, data, formula ))  
+    return(hrm.test.1.none(X, alpha , group, subject, data, formula, nonparametric ))  
   
   }
   
@@ -416,7 +532,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.one(X, alpha, group , factor1, subject, data, testing, formula ))   
+    return(hrm.test.2.one(X, alpha, group , factor1, subject, data, testing, formula, nonparametric ))   
   }
   
   # Case: 2 wholeplot, 1 subplot factor
@@ -463,7 +579,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.between(X, alpha, group , subgroup, factor1, subject, data, testing, formula )) 
+    return(hrm.test.2.between(X, alpha, group , subgroup, factor1, subject, data, testing, formula, nonparametric )) 
   }
   
   # Case: 1 wholeplot, 2 subplot factors
@@ -508,7 +624,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.within(X, alpha, group , factor1, factor2, subject, data, testing, formula ))
+    return(hrm.test.2.within(X, alpha, group , factor1, factor2, subject, data, testing, formula, nonparametric ))
   }
   
   # Case: 1 wholeplot, 3 subplot factors
@@ -597,7 +713,7 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.3.between(X, alpha, group , factor1, factor2, factor3, subject, data, testing, formula ))
+    return(hrm.test.3.between(X, alpha, group , factor1, factor2, factor3, subject, data, testing, formula, nonparametric ))
   }
   
   
@@ -678,6 +794,6 @@ hrm_test_internal <- function(formula, data, alpha = 0.05,  subject ){
     }
     X<-data
     data <- colnames(dat)[1]
-    return(hrm.test.2.between.within(X, alpha, group , subgroup, factor1, factor2, subject, data, testing, formula ))
+    return(hrm.test.2.between.within(X, alpha, group , subgroup, factor1, factor2, subject, data, testing, formula, nonparametric ))
   }
 }
\ No newline at end of file
