@@ -19,9 +19,9 @@
 #' @param text a string, which will be printed in the output
 #' @return Returns a data frame consisting of the degrees of freedom, the test value, the critical value and the p-value
 #' @keywords internal
-hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text = "" ){
+hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text = "" , nonparametric, ranked){
   
-  stopifnot(is.data.frame(X),is.character(subject), is.character(group),is.character(factor1),is.character(factor2), alpha<=1, alpha>=0)
+  stopifnot(is.data.frame(X),is.character(subject), is.character(group),is.character(factor1),is.character(factor2), alpha<=1, alpha>=0, is.logical(nonparametric))
   
   f <- 0
   f0 <- 0
@@ -33,20 +33,58 @@ hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text
   factor1 <- as.character(factor1)
   factor2 <- as.character(factor2)
   subject <- as.character(subject)
+  
+  
+  X <- as.data.table(X)
+  setnames(X, c(data, group, factor1, factor2, subject), c("data", "group", "factor1", "factor2", "subject"))
+  
+  a <- nlevels(X[,group])
+  d <- nlevels(X[,factor1])
+  c <- nlevels(X[,factor2])
+  n <- table(X[,group])/(d*c)
+  KGV <- Reduce(Lcm, n)
+  lambda <- KGV/n
+  
+  
+  if(max(lambda) <= 100 & max(n) <= 30 & nonparametric & is.null(ranked)){
+    
+    len <- dim(X)[1]
+    prData <- list(X,0)
+    z <- levels(X[,group])
+    
+    # amplify data to artificially create balanced groups
+    for(i in 1:a){
+      prData[[i+1]] <- X[group==z[i]][rep(1:(n[i]*d*c), each = (lambda[i]-1)), ]
+    }
+    X <- rbindlist(prData)
+    X[,data]<- (rank(X[,data], ties.method = "average")-1/2)*1/(KGV*a*d*c)
+    
+    # select original observations from amplified data
+    X <- X[1:len,]
+  }
+  
+
   X <- split(X, X[,group], drop=TRUE)
-  a <- length(X)
-  d <- nlevels(X[[1]][,factor1])
-  c <- nlevels(X[[1]][,factor2])
-  n <- rep(0,a) 
-  
-  
   for(i in 1:a){
-    X[[i]] <- X[[i]][ order(X[[i]][,subject], X[[i]][,factor1], X[[i]][,factor2]), ]
-    X[[i]]<- X[[i]][,data]
+    X[[i]] <- X[[i]][ order(subject, factor1, factor2), ]
+    X[[i]] <- X[[i]][,data]
     X[[i]] <- matrix(X[[i]],ncol=d*c,byrow=TRUE)
     n[i] <- dim(X[[i]])[1]
   }
-  
+
+  if((max(lambda) > 100 | max(n) > 30) & nonparametric & is.null(ranked)){
+    X <- pseudorank(X)
+    for(i in 1:a){
+      X[[i]] <- 1/(sum(n)*d*c)*(X[[i]] - 1/2)
+    }
+  }
+
+  if(is.null(ranked)){
+    eval.parent(substitute(ranked<-X))
+  } else {
+    X <- ranked
+  }
+
   # creating X_bar (list with a entries)
   X_bar <- as.matrix(vec(sapply(X, colMeans, na.rm=TRUE)))
   
@@ -85,6 +123,13 @@ hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text
   K_Hypothesis <- kronecker(S, K)
   V <- lapply(X, DualEmpirical2, B=K)
   
+  Q = data.frame(Q1 = rep(0,a), Q2 = rep(0,a))
+  if(nonparametric){
+    for(i in 1:a){
+      Q[i,] <- calcU(X,n,i,K)
+    }    
+  }
+  
   #################################################################################################
   
   # f
@@ -92,7 +137,7 @@ hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text
   f_2 <- 0
   
   for(i in 1:a){
-    f_1 <- f_1 + (S[i,i]*1/n[i])^2*.E1(n,i,V[[i]])
+    f_1 <- f_1 + (S[i,i]*1/n[i])^2*.E1(n,i,V[[i]], nonparametric, Q)
     j <- i+1
     while(j<=a){
       f_1 <- f_1 + 2*(S[i,i]*1/n[i])*(S[j,j]*1/n[j])*.E3(V[[i]],V[[j]])
@@ -101,7 +146,7 @@ hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text
   }
   
   for(i in 1:a){
-    f_2 <- f_2 + (S[i,i]*1/n[i])^2*.E2(n,i,V[[i]])
+    f_2 <- f_2 + (S[i,i]*1/n[i])^2*.E2(n,i,V[[i]], nonparametric, Q)
     j <- i+1
     while(j<=a){
       f_2 <- f_2 + 2*S[i,j]*S[j,i]*1/(n[i]*n[j])*.E4(1/(n[i]-1)*P(n[i])%*%X[[i]],1/(n[j]-1)*K%*%t(X[[j]])%*%P(n[j])%*%X[[j]]%*%K%*%t(X[[i]])%*%P(n[i]))
@@ -123,7 +168,7 @@ hrm.1w.2f <- function(X, alpha, group , factor1, factor2, subject, data, H, text
   
   
   for(i in 1:a){
-    f0_2 <- f0_2 + (S[i,i]*1/n[i])^2*1/(n[i]-1)*.E2(n,i,V[[i]])
+    f0_2 <- f0_2 + (S[i,i]*1/n[i])^2*1/(n[i]-1)*.E2(n,i,V[[i]], nonparametric, Q)
   }
   
   f0 <- f0_1/f0_2
